/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg.springboottestcontainers;

import java.util.List;
import java.util.Optional;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.boot.testcontainers.service.connection.ServiceConnection;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.testcontainers.containers.PostgreSQLContainer;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 *
 * @author omozegieaziegbe
 */
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@Testcontainers
public class ProductControllerTest {

    @LocalServerPort
    private Integer port;

    @Autowired
    private TestRestTemplate restTemplate;

    @Autowired
    private ProductRepository productRepository;

    @Container
    @ServiceConnection
    static PostgreSQLContainer<?> postgres = new PostgreSQLContainer<>(
            "postgres:15-alpine"
    );

    @Test
    public void testProductEndPoints() {

        // Create new product
        Product product = new Product();
        product.setName("Real World Java EE Patterns");
        
        ResponseEntity<Product> createResponse =
                restTemplate.postForEntity("/api/products", product, Product.class);
        assertEquals(HttpStatus.OK, createResponse.getStatusCode());
        Product savedProduct = createResponse.getBody();
        productRepository.save(savedProduct);
        
        // Retrieve All products
        ResponseEntity<Product[]> getAllResponse =
                restTemplate.getForEntity("/api/products", Product[].class);
        assertEquals(HttpStatus.OK, getAllResponse.getStatusCode());
        
        Product[] getAllProducts = getAllResponse.getBody();
        assert getAllProducts != null;
        assertEquals(1, getAllProducts.length);

    }

}
